(*****************************************************************************
*** Name of the program:   Serial_To_I2C
*** Function:              I2C-Interface at the serial port
*** Author:                Andreas Hesse
*** Date of construction:  2002-01-24
*** Date of last revision: 2004-06-20
*** Version:               1.0.3.2
*** Comment:
***
***
***
******************************************************************************
***
*** + = Adding new feature
*** - = Bugfix
*** * = Changed
*** ? = detected problem
***
*** Version management:
*** V 1.0.3.2   * Even if no interface connected, user can check a checkbox
***               to allow actions (for test purposes)
*** V 1.0.3.1   + Temperature Measurement - Circuit LM75 implemented
*** V 1.0.3.0   + Display SpecialSigns Value to easly find configuration
***             ? (3) Failure in interface will not be cleared
***             ? (2) At startup COM-Combobox does not work correctly
*** V 1.0.2.0   * Implementation of Display finished
***               Note: Not all Functions of display-driver are implemented
***               but most of them applicable to DSM-0822A.
***             - New SIO-Function "CheckCable" checks the correct wiring before
***               start of the different connections
***             + If no cable detected, it will not be possible to start any
***               action concerning the I2C-Circuits (Tabsheets are freezed)
*** V 1.0.1.1   + Implementation of Display
*** V 1.0.1.0   * Implementation of 8591 finished
***             + Implementation of EEProm
*** V 1.0.0.1   + Implementation of new sio functions
***             + Implementation of 8591 (started)
***             ? (1) If there is no interface connected, the program finds 128 slaves.
*** V 1.0.0.0   First construction based on several sources of the internet
***             Thanks to:
***             Robert-Bosch-Schule Ulm
***             Abt. Elektrotechnik, Datentechnik, Programmiersprachen
***
*** *)

unit Serial_To_I2C;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, i2csiou, ComCtrls, HLEdit, ToolWin, ComboBox_SAVE,
  COMCB, ImgList, Menus, Filehandling, ExtCtrls, MyMath, inifiles, String_Functions,
  HLLabeledEdit, Digitalanzeige,I2CDisplay, CheckLst;

const delaytime=10;
      MaxDisplayChar=8; (* length of display *)

type
  PIOPoint = ^TIOPoint;
  TIOPoint = Record
    Output:Boolean;
    Name:String;
    PIN:Integer;
  end;

  TFO_I2C = class(TForm)
    PC1: TPageControl;
    TS_IO1: TTabSheet;
    TS_ADDA1: TTabSheet;
    TS_System: TTabSheet;
    GB_IO1: TGroupBox;
    SB1: TStatusBar;
    PB_SendtoIO1: TButton;
    PB_ReadFromIO1: TButton;
    LV_Sys: TListView;
    TB1: TToolBar;
    PB_Lookfordevices: TButton;
    SB_sys: TStatusBar;
    IL1: TImageList;
    LV_IO1: TListView;
    PM_IO1: TPopupMenu;
    ypndern1: TMenuItem;
    MainMenu1: TMainMenu;
    Datei1: TMenuItem;
    IO11: TMenuItem;
    ypndern2: TMenuItem;
    CB_AdrIO1: TComboBox_SAVE;
    Label2: TLabel;
    MI_Save: TMenuItem;
    MI_NewIO: TMenuItem;
    SB_ADDA1: TStatusBar;
    GB_ADDA1: TGroupBox;
    RG_ainadda1: TRadioGroup;
    PB_RW_Analog: TButton;
    IOPunkt1: TMenuItem;
    MI_IOPoint_Name: TMenuItem;
    TS_EEPROM: TTabSheet;
    Label5: TLabel;
    GroupBox1: TGroupBox;
    CB_ADRADDA1: TComboBox;
    Label1: TLabel;
    Label3: TLabel;
    ED_UrefADDA1: THLEdit;
    Label6: TLabel;
    CB_AutoIncADDA1: TCheckBox;
    GroupBox2: TGroupBox;
    Label4: TLabel;
    CB_AoutAdda1: TCheckBox;
    ED_UoutADDA1: THLEdit;
    CB_EEPROM_ADR: TComboBox_SAVE;
    COMCB1: TCOMCB;
    PB_CheckCable: TButton;
    PB_ReadEEProm: TButton;
    ME_EEProm: TMemo;
    PB_WriteEEProm: TButton;
    ED_EEPROMLength: THLLabeledEdit;
    SD1: TSaveDialog;
    PB_SaveContentsOfEEPROM: TButton;
    RG_EEPromFormat: TRadioGroup;
    HLEdit1: THLEdit;
    RG_ADDA1_AIN: TGroupBox;
    Label7: TLabel;
    DA1: TDigitalanzeige;
    Label8: TLabel;
    DA2: TDigitalanzeige;
    Label9: TLabel;
    DA3: TDigitalanzeige;
    Label10: TLabel;
    DA4: TDigitalanzeige;
    HeaderControl1: THeaderControl;
    TS_Display: TTabSheet;
    RG_DisplayBlink: TRadioGroup;
    GroupBox3: TGroupBox;
    CLB_SpecialSigns1: TCheckListBox;
    CLB_SpecialSigns2: TCheckListBox;
    GroupBox4: TGroupBox;
    CB_AdrDisplay: TComboBox_SAVE;
    Label11: TLabel;
    PB_SendToDisplay: TButton;
    ED_LCD_Displaytext: THLLabeledEdit;
    RG_DriveMode: TRadioGroup;
    RG_Bias: TRadioGroup;
    RG_Display: TRadioGroup;
    RG_PowerMode: TRadioGroup;
    RG_BlinkMode: TRadioGroup;
    Label12: TLabel;
    IL2: TImageList;
    SB_IO1: TStatusBar;
    LB_Specialsignval1: TLabel;
    LB_Specialsignval2: TLabel;
    est1: TMenuItem;
    Anschalten1: TMenuItem;
    TS_Temperature: TTabSheet;
    DA_Temp: TDigitalanzeige;
    GroupBox5: TGroupBox;
    CB_AdrTemp: TComboBox_SAVE;
    Label13: TLabel;
    PB_ReadTemp: TButton;
    Panel1: TPanel;
    ED_AlarmTemp: THLLabeledEdit;
    PB_ReadAlarmTemp: TButton;
    PB_AlarmTempWrite: TButton;
    Panel2: TPanel;
    ED_ClearTemp: THLLabeledEdit;
    PB_ReadTHyst: TButton;
    PB_WriteTHyst: TButton;
    GroupBox6: TGroupBox;
    CLB_TempConfig: TCheckListBox;
    RG_TempFaultQueue: TRadioGroup;
    PB_WriteTempConfig: TButton;
    CB_ContTempRead: TCheckBox;
    TI_TemperatureRead: TTimer;
    PopupMenu1: TPopupMenu;
    CB_RunWithoutCheck: TCheckBox;
    Panel3: TPanel;
    Label14: TLabel;
    LB_Configbyte: TLabel;
    Label15: TLabel;
    Label16: TLabel;
    LB_AlarmLSB: TLabel;
    LB_AlarmMSB: TLabel;
    Label17: TLabel;
    Label18: TLabel;
    LB_THYSTMSB: TLabel;
    LB_THYSTLSB: TLabel;
    procedure PB_LookfordevicesClick(Sender: TObject);
    procedure PB_SendtoIO1Click(Sender: TObject);
    procedure PB_ReadFromIO1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure LV_IO1ColumnClick(Sender: TObject; Column: TListColumn);
    procedure LV_IO1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure ypndern1Click(Sender: TObject);
    procedure MI_SaveClick(Sender: TObject);
    procedure MI_NewIOClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure LV_IO1Click(Sender: TObject);
    procedure LV_IO1Changing(Sender: TObject; Item: TListItem;
      Change: TItemChange; var AllowChange: Boolean);
    procedure PB_RW_AnalogClick(Sender: TObject);
    procedure MI_IOPoint_NameClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure Load_Config;
    procedure Save_Config;
    procedure PB_CheckCableClick(Sender: TObject);
    procedure PB_ReadEEPromClick(Sender: TObject);
    procedure PB_WriteEEPromClick(Sender: TObject);
    procedure PB_SaveContentsOfEEPROMClick(Sender: TObject);
    procedure PB_SendToDisplayClick(Sender: TObject);
    procedure CLB_SpecialSigns1Click(Sender: TObject);
    procedure Anschalten1Click(Sender: TObject);
    procedure PB_ReadTempClick(Sender: TObject);
    procedure PB_WriteTempConfigClick(Sender: TObject);
    procedure PB_ReadAlarmTempClick(Sender: TObject);
    procedure PB_ReadTHystClick(Sender: TObject);
    procedure PB_AlarmTempWriteClick(Sender: TObject);
    procedure PB_WriteTHystClick(Sender: TObject);
    procedure CB_ContTempReadClick(Sender: TObject);
    procedure TI_TemperatureReadTimer(Sender: TObject);
    procedure CB_RunWithoutCheckClick(Sender: TObject);
  private
    { Private declarations }
    procedure FillIO1LV(LVX:TListview);
    procedure sendbytetoIO(adr,dat:Integer);
    Procedure LoadIO1def(Filename:String);
    Function newelement(Namex:String;No:integer;Neu:Boolean):Boolean;
    Procedure CalcSpecialSigns(var byte1,byte2:Integer);
    Procedure SetDesiredTemp (TempType:Integer; ValueToWrite:Integer);
    function CalcLM75ConfigByte:Integer;
    Procedure Tabsheet_Handling(bEnable:Boolean);
  public
    { Public declarations }
  end;

var
  FO_I2C: TFO_I2C;


implementation

{$R *.dfm}

(* Configuration *)

procedure TFO_I2C.Save_Config;
Var inidat:TInifile;
    verz:string;
begin
  verz:=ExtractFilePath(Application.ExeName);
  inidat:=Tinifile.create(verz+'I2CWIN.ini');
  inidat.writeinteger('Configuration','COMPort',COMCB1.ItemIndex+1);
  inidat.writeinteger('View','Left',FO_I2C.Left);
  inidat.writeinteger('View','Top',FO_I2C.Top);
  inidat.writeinteger('View','Width',FO_I2C.Width);
  inidat.writeinteger('View','Height',FO_I2C.Height);
  Inidat.writeinteger('ADDA','AIConfig',RG_ainadda1.ItemIndex);
  inidat.writebool('ADDA','AOut',CB_AoutAdda1.checked);
  Inidat.writestring('ADDA','UREF',ED_UrefADDA1.text);
  Inidat.writestring('ADDA','UOUT',ED_UoutADDA1.text);
  inidat.writebool('ADDA','Autoincrement',CB_AutoIncADDA1.checked);
  inidat.free;
end;

procedure TFO_I2C.load_config;
Var inidat:TInifile;
    verz:string;
begin
  verz:=ExtractFilePath(Application.ExeName);
  inidat:=Tinifile.create(verz+'I2CWIN.ini');
  COMCB1.ItemIndex :=
    Inidat.readinteger('Configuration','COMPort',COMCB1.ItemIndex)-1;
  FO_I2C.Left := inidat.readinteger('View','Left',FO_I2C.Left);
  FO_I2C.Top := inidat.readinteger('View','Top',FO_I2C.Top);
  FO_I2C.Width := inidat.readinteger('View','Width',FO_I2C.Width);
  FO_I2C.Height := inidat.readinteger('View','Height',FO_I2C.Height);
  RG_ainadda1.ItemIndex :=
    Inidat.readinteger('ADDA','AIConfig',RG_ainadda1.ItemIndex);
  CB_AoutAdda1.checked :=
    inidat.readbool('ADDA','AOut',CB_AoutAdda1.checked);
  ED_UrefADDA1.text :=
    Inidat.readstring('ADDA','UREF',ED_UrefADDA1.text);
  ED_UoutADDA1.text :=
    Inidat.readstring('ADDA','UOUT',ED_UoutADDA1.text);
  CB_AutoIncADDA1.checked :=
    inidat.readbool('ADDA','Autoincrement',CB_AutoIncADDA1.checked);
  inidat.free;
end;

Function TFO_I2C.newelement(Namex:String;No:integer;Neu:Boolean):Boolean;
  var AT:TTabsheet;

  procedure NewListview;
    Var tmplc:TListColumn;
        i:Integer;
        LI:TListItem;
        LVtmp:TListView;
  begin
    LVtmp:=TListview.Create(AT);    (* New Listview ... *)
    with LVtmp do
    begin
      Parent:=AT;               (* ... belongs to Pagecontrol *)
      Name:='LVX_'+IntToStr(No); (* ... und erhlt Namen: ed_1, ed_2, ... *)
      font.name:='MS Sans Serif';
      font.size:=8;       {}
      align:=alclient;
      //LVtmp.ScrollBars := ssvertical;
      ViewStyle:=vsReport;
      SmallImages:=IL1;
      LVtmp.Checkboxes:=True;
      tmplc := Columns.Add;
      tmplc.Caption := 'Nr';
      tmplc := Columns.Add;
      tmplc.Caption := 'Name';
      tmplc.Width := 250;
      tmplc := Columns.Add;
      tmplc.Caption := 'Pin';
      tmplc.Width := 50;
      for i:=1 to 8 do
      begin
        LI:=LVtmp.Items.Add;
        LI.Caption:=inttostr(i);
        LI.SubItems.Add('Bit'+inttostr(i));
        LI.SubItems.Add('Ausgang');
        LI.ImageIndex:=1;
      end
    end
  end;  //NewListview
begin
  //if ((not neu) and fileexists(HC_1.sections[0].text+Notizname+'.txt'))
    //or neu  then
    with Pc1 do
    begin
      AT:=ttabsheet.create(self);
      with AT do
      begin { Eigenschaften setzen       {}
        pagecontrol:=PC1;
        // define unique name
        name:='tabs_'+IntToStr(No);
        caption:=Namex;
        Pc1.activepage:=AT;
        NewListview;
        Brush.style:=bssolid;
        newelement:=true;

     end   { With Tabsheet       {}
   end    { With PC_Notizen      {}
  //else
    //  showmessage('Es trat ein Fehler beim laden der Datei '+Notizname+'.txt auf!');
end;

Procedure TFO_I2C.LoadIO1def(Filename:String);
  { var delimiter,textdelimiter,CH_read,lastchar:Char;
      fieldcounter,rowcounter,ij:integer;
      strh1:string;
      F:TextFile;   }
begin
  {delimiter:=',';
  textdelimiter:='"';
  Rowcounter:=0;
  //lastchar:=chr(0);  }
end;

function CalcTemperature (msb,lsb:Integer):Real;
  var tmpTemp:Real;
Begin
  if msb < 128 then   (* Positive temperature *)
  Begin
    tmpTemp:=msb;
    If lsb>127 Then
      tmpTemp:=tmpTemp + 0.5;
  end (* if msb.. *)
  else
    Begin
      tmpTemp:=msb - 128;
      tmpTemp:=0-tmpTemp;
    end;
  CalcTemperature:=tmpTemp;
End;  (* Function CalcTemperature *)

Procedure TFO_I2C.Tabsheet_Handling(bEnable:Boolean);
  var ix:integer;
Begin
  for ix:=0 to PC1.PageCount-1 do
    if PC1.Pages[ix].Caption<>'System' then
    Begin
       PC1.Pages[ix].Enabled:=bEnable;
       if bEnable then
        PC1.Pages[ix].ImageIndex:=0
       else
        PC1.Pages[ix].ImageIndex:=1
    end
end;

procedure TFO_I2C.PB_LookfordevicesClick(Sender: TObject);

  var i:byte;
      jx,devno,ix:Integer;
      ListItem:TListItem;
      found:boolean;

begin
  if CheckCable then
  Begin
    Tabsheet_Handling(True); (* Enable Tabsheets *)
    SB1.Color:=clBtnFace;
    SB1.Panels[0].Text:='Anschluss gefunden!';
    devno:=0;
    found:=False; (* To supress compiler warning *)
    LV_Sys.Items.Clear;
    CB_AdrIO1.Items.Clear;
    jx:=DelayTime;
    for i:=0 to 255 do
    begin
      iic_start(jx);
      iic_w(i);
      if iic_ack and (i mod 2=0) then
      (* I mod 2 filters the following: any I2C-Bus Circuit has an adressnumber
         where the last bit indicates Read or write  *)
      begin
        devno:=devno+1;
        ListItem := LV_Sys.Items.Add;
        ListItem.Caption := inttostr(devno);
        for ix:=0 to (I2CDef.count-1) do
        begin
          found:=false;
          I2CDefrec:=I2CDef.Items[ix];
          If (i>=I2CDefrec^.lowadr) and (i<=I2CDefrec^.Highadr) then
          begin
            found:=true;
            Case i of
              64..78:
                if i mod 2 = 0 then
                begin
                  CB_AdrIO1.Items.Add(Hex(i));
                  CB_AdrIO1.ItemIndex:=0;
                  found:=true;
                end;
              112:
                Begin
                  CB_AdrDisplay.Items.add(Hex(i));
                  CB_AdrDisplay.ItemIndex:=0;
                  found:=true;
                end;
              144..157:
                Begin
                  if i mod 2 = 0 then
                  begin
                    CB_ADRADDA1.Items.Add(Hex(i));
                    CB_ADRADDA1.ItemIndex:=0;
                  end
                end;
              158..159:
                Begin
                  if i mod 2 = 0 then
                  begin
                    CB_AdrTemp.Items.Add(Hex(i));
                    CB_AdrTemp.ItemIndex:=0;
                  end
                end
            end; (* case *)
            ListItem.subitems.add(I2CDefrec^.name);
            ListItem.subitems.add(IntToStr(i));
            ListItem.subitems.add(Hex(i));
          end;  {}
        end;  (* For *)
        if not found then
        begin
          I2CDefrec:=I2CDef.Items[I2CDef.count-1];
          (* take last item as default value for unknown devices *)
          ListItem.subitems.add(I2CDefrec.name);
          ListItem.subitems.add(inttostr(i));
          ListItem.subitems.add(Hex(i));
        end;
      end;  (* IIC_ACK *)
      iic_stop;
    end;
    if devno=0 then
    begin
      SB_sys.Font.Color:=ClBlack;
      SB_sys.Color:=clred;
      SB_sys.Panels[0].Text:='Keinen Slave gefunden!';
    end
    else
      if devno=128 then
      begin
        SB_sys.Color:=clred;
        SB_sys.Panels[0].Text:='Mglicherweise ist kein Kabel angeschlossen!';
      end
      else
      begin
        SB_sys.Color:=clBtnFace;
        SB_sys.Panels[0].Text:='Suche war erfolgreich!';
      end
  end (* CheckCable *)
  else
  Begin
    SB1.Color:=clred;
    SB1.Panels[0].Text:='Kabel defekt oder keine Spannung an Slave-Bausteinen!';
    Tabsheet_Handling(False); (* Disable Tabsheets *)
  end;
end;

{
procedure TProjectForm.PageControlDrawTab(Control: TCustomTabControl;
TabIndex: Integer; const Rect: TRect; Active: Boolean);
var
  R: TRect;
  Sheet: TTabSheet;
begin
  R := Rect;
  with PageControl, ApplicationData.SystemImages do
  begin
    Sheet := Pages[TabIndex];
    Inc(R.Left, 2);
    if Assigned(Images) and (Sheet.ImageIndex > -1) then
    begin
      if Active then
      begin
        Inc(R.Left, 2);
        Draw(Control.Canvas, R.Left, R.Top + 4, Sheet.ImageIndex);
      end
      else Draw(Control.Canvas, R.Left, R.Top + 2, Sheet.ImageIndex);
      Inc(R.Left, Width);
    end;
    if Active then Canvas.Font.Color := clBlack
              else Canvas.Font.Color := clBtnShadow;
    DrawText(Control.Canvas.Handle, PChar(Sheet.Caption),
Length(Sheet.Caption), R,
             DT_CENTER or DT_VCENTER or DT_SINGLELINE);
  end;
end;}

procedure TFO_I2C.sendbytetoIO(adr,dat:Integer);
begin
  iic_start(DelayTime);
  iic_w(adr);
  if iic_ack then
  begin
    iic_w(dat);
    if iic_ack then
      SB1.Panels[0].Text:='OK'
    else
      SB1.Panels[0].Text:='FEHLER beim Schreiben der Daten';
    iic_stop;
  end
  else
  begin
    SB1.Panels[0].Text:='FEHLER beim Schreiben der Adresse';
    iic_stop;
  end;
end;  (* sendbytetoIO *)

procedure TFO_I2C.PB_SendtoIO1Click(Sender: TObject);
  var err,i,adr,dat:integer;
begin
  val('$'+CB_AdrIO1.Text,adr,err);
  dat:=0;
  For i:=0 to 7 do
  Begin
    if (LV_IO1.Items[i].ImageIndex=1) and LV_IO1.Items[i].Checked then
      dat:=dat+zweierpotenz(i)
  end;
  sendbytetoIO(adr,dat);
end;

procedure TFO_I2C.PB_ReadFromIO1Click(Sender: TObject);
  var Data:Byte;
      err,adr,i:integer;
begin
  Data:=0; (* CWA *)
  val('$'+CB_AdrIO1.Text,adr,err);
  iic_start(DelayTime);
  iic_w(adr);
  if iic_ack then
  begin
    iic_w($FF);
    if iic_ack then
    begin
      SB1.Panels[0].Text:='OK';
      iic_stop;
      iic_start(DelayTime);
      iic_w(adr+1);
      if iic_ack then
      begin
        data:=iic_rack;
        iic_stop;
      end
      else
        SB1.Panels[0].Text:='FEHLER beim Schreiben der Daten';
    end
    else
    begin
      SB1.Panels[0].Text:='FEHLER beim Schreiben der Daten';
      iic_stop;
    end
  end
  else
  begin
    SB1.Panels[0].Text:='FEHLER beim Schreiben der Adresse';
    iic_stop;
  end;
  PB_SendtoIO1Click(Sender);
  For i:=0 to 7 do
  Begin
    if (LV_IO1.Items[i].ImageIndex=0) then
      if (data and zweierpotenz(i+1))=zweierpotenz(i+1) then
        LV_IO1.Items[i].Checked:=false
      else
        LV_IO1.Items[i].Checked:=true
  end;
  showmessage(inttostr(data));
end;

procedure TFO_I2C.FormCreate(Sender: TObject);
  var verz:string;
    i:integer;
    a:boolean;
begin
  Load_Config;
  {if not InStr(COMCB1.text,'belegt', false) then
    OpenCom(COMCB1.ItemIndex+1)
  else
    showmessage('COM' + inttostr(COMCB1.ItemIndex+1) +' ist bereits belegt!');
             {}
  OpenCom(1);
  IOs := TList.Create;
  I2CDef := TList.Create;
  verz:=ExtractFilePath(Application.ExeName);
  LoadIOs(verz+'IOs.txt',IOs);
  a:=LoadI2Cdef(verz+'I2CDef.txt',I2Cdef);
  PB_LookfordevicesClick(Sender);
  if I2CDef.Count>0 then
    for i := 0 to (I2CDef.Count - 1) do
    begin
      I2CDefrec := I2CDef.Items[i];
      // showmessage(I2CDefrec.Name+':'+inttostr(I2CDefrec.lowadr)+'-'+inttostr(I2CDefrec.highadr));
    end;
  FillIO1LV(LV_IO1);
  SetDisplayChars
end;

procedure TFO_I2C.FormClose(Sender: TObject; var Action: TCloseAction);
  var ix:Integer;
begin
  for ix := 0 to (I2CDef.Count - 1) do
  begin
    I2CDefrec := I2CDef.Items[ix];
    Dispose(I2CDefrec);
  end;
  I2CDef.Free;
  closecom;
  Save_Config;
  //I2CDef.Free;
  //IOs,I2CDef:TLIST;
  //IOlist:PIO;
  //I2CDefrec:PI2CDef;
end;

procedure TFO_I2C.LV_IO1ColumnClick(Sender: TObject; Column: TListColumn);
begin
  // if column.Caption='Typ' then
 // showmessage(column.Caption);
end;

Procedure TFO_I2C.FillIO1LV(LVX:TListview);
  var i:Integer;
      LI:TListItem;
Begin
  for i:=1 to 8 do
  begin
    LI:=LVX.Items.Add;
    LI.Caption:=inttostr(i);
    if IOs.count > i then
    begin
      IoList:=IOs.Items[i-1];
      LI.SubItems.Add(IOList^.Name);
      //LI.SubItems.Add('Ausgang');
    end
    else
    begin
      LI.SubItems.Add('Bit'+inttostr(i));
      LI.SubItems.Add('Ausgang');
      LI.ImageIndex:=1;
    end;
  end;
end;

procedure TFO_I2C.LV_IO1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
 // var curpos:TPoint;
   //   actItem:Tlistitem;
begin
  {showmessage(sender.ClassName);
  with sender as TlistView do
    showmessage(GetItemAt(X, Y).Caption);


  if button=mbRight then
    PM_IO1.Popup(x+FO_I2C.Left+LV_IO1.Left,y+FO_I2C.Top+LV_IO1.Top+30);

 { s with sender as TlistView do
  begin
    actitem:=TListItem.create(Items);
    curpos.X:=x+FO_I2C.Left+LV_IO1.Left;
    curpos.Y:=y+FO_I2C.Top+LV_IO1.Top;;
    actItem:=GetNearestItem(curpos, sdall);
    showmessage(actitem.Caption);
  end;


{    With sender as TListview do
    Begin
      GetCursorPos(curpos);
      actitem:=TListItem.create(Items);

      //showmessage(selected.Caption);
      {curpos.X:=curpos.X-left;
    curpos.Y:=curpos.Y-top;
    curpos.X:=70;
    curpos.Y:=20;
    actItem:=GetNearestItem(curpos, sdall);
    showmessage(inttostr(curpos.X));
    if actitem<>Nil then
      showmessage(actitem.Caption)
    else
      showmessage('*No')
    {sdLeft ,sdRight, sdAbove  sdBelow	 sdAll      {}
   // end
  //end;
end;

procedure TFO_I2C.ypndern1Click(Sender: TObject);
begin
  With LV_IO1 do
    if selected <> nil then
    begin
      if selected.ImageIndex=0 then
      begin
        selected.ImageIndex:=1;
        Selected.SubItems[1]:='Ausgang';
      end
      else
      begin
        selected.SubItems[1]:='Eingang';
        selected.ImageIndex:=0;
      end
    end
    else
      SB1.Panels[0].Text:='Keine PIN angewhlt!';
end;

procedure TFO_I2C.MI_SaveClick(Sender: TObject);
  var verz:String;
begin
  verz:=ExtractFilePath(Application.ExeName);
  SaveIOs(verz+'IOs.txt',IOs);
end;

procedure TFO_I2C.MI_NewIOClick(Sender: TObject);
  var i:integer;
begin
  New(IOlist);
  IOlist^.Adress:=0;
  IOList^.Name:='IO'+Inttostr(Ios.Count+1);
  IOs.Add(IOlist);
  if newelement(IOList^.Name,IOs.count,true) then
     i:=0;
  dispose(iolist);
end;

procedure TFO_I2C.LV_IO1Click(Sender: TObject);
begin
//showmessage('test');
end;

procedure TFO_I2C.LV_IO1Changing(Sender: TObject; Item: TListItem;
  Change: TItemChange; var AllowChange: Boolean);
begin
  {if (item.ImageIndex=0) and (change=ctstate) then
    AllowChange:=false
  else
    AllowChange:=True;    {}
end;

procedure TFO_I2C.PB_RW_AnalogClick(Sender: TObject);
var err:integer;
    adr,mode,dau,k1,k2,k3,k4: byte;

  Function CalcMode:Integer;
  (* Calc the controlbyte *)
    var mi,ix:Integer;
  Begin
    mi:=0;
    if CB_AoutAdda1.Checked then
      mi:=mi+zweierpotenz(6);
    ix:=RG_ainadda1.ItemIndex;
    if (ix > 2) then
    (* set bit 5 *)
      mi:=mi+zweierpotenz(5);
    if (ix mod 2<>0) then
    (* set bit 4 *)
      mi:=mi+zweierpotenz(4);
    if CB_AutoIncADDA1.Checked then
      mi:=mi+zweierpotenz(2);
    Result:=mi;
  end;

  Function CalcDAU:Integer;
  (* Calc the Analog OUT *)
    var mi:Integer;
        uref,uout:Real;
  Begin
    //mi:=0;
    uref:=StrToFloatDef(ED_UrefADDA1.Text,0);
    uout:=StrToFloatDef(ED_UoutADDA1.Text,0);
    if uout>uref then
    Begin
      mi:=0;
      Showmessage('Fehler: Ausgangsspannung ist grsser als Referenzspannung!');
    end
      else
        Begin
          mi:=Round(uout*256/Uref)-1; // Calc ???
        end;
    Result:=mi;
  end;


begin
  val('$'+CB_ADRADDA1.Text,adr,err);
  mode:=CalcMode;
  //showmessage(inttostr(mode));
  dau:=CalcDAU;
  iic_start(Delaytime);
  iic_w(adr);
  if  not iic_ack then
  begin
    SB_ADDA1.Panels[0].Text:='FEHLER beim Schreiben der Adresse';
    iic_stop;
  end
  else
  Begin
    iic_w(mode);
    if not iic_ack then
    begin
      SB_ADDA1.Panels[0].Text:='FEHLER beim Schreiben des Mode-Byte';
      iic_stop;
    end
    else
    Begin
      iic_w(dau);
      if  not iic_ack then
      begin
        SB_ADDA1.Panels[0].Text:='FEHLER beim Schreiben des Analogout-Wertes';
        iic_stop;
      end;
      iic_stop;
    end;
  end;  (* else Mode *)  {}

  (* Byteorder of 8591:
    adress
    Mode
    Analog Out
    *)
  iic_start(Delaytime);
  iic_w(adr+1);
  if  not iic_ack then
  begin
    SB_ADDA1.Panels[0].Text:='FEHLER Kein ACK fuer Lesen von Adr '+hex(adr+1);
    iic_stop;
  end
  else
  Begin
    k1:=iic_rack(); (* Do not use first value *)
    k1:=iic_rack();
    k2:=iic_rack();
    k3:=iic_rack();
    k4:=iic_rnak();
    iic_stop;
    SB_ADDA1.Panels[0].Text:='OK '+hex(k1)+' '+hex(k2)+' '+hex(k3)+' '+hex(k4);
    DA1.Value:=IntToStr(k1);
    DA2.Value:=IntToStr(k2);
    DA3.Value:=IntToStr(k3);
    DA4.Value:=IntToStr(k4);
  end;
end;

procedure TFO_I2C.MI_IOPoint_NameClick(Sender: TObject);
  var newname:String;
begin
  With LV_IO1 do
    if selected <> nil then
    begin
      newname:=InputBox('I-O-Punkt: ', 'Neuer Name:', selected.SubItems[0]);
      if newname<>'' then
        selected.SubItems[0]:=newname;

      {if selected.ImageIndex=0 then
      begin
        selected.ImageIndex:=1;
        Selected.SubItems[1]:='Ausgang';
      end
      else
      begin
        selected.SubItems[1]:='Eingang';
        selected.ImageIndex:=0;
      end  }
    end
    else
      SB1.Panels[0].Text:='Keine PIN angewhlt!';
end;

procedure TFO_I2C.FormShow(Sender: TObject);
const
  breite=800; {Auflsung, unter der das Fenster erstellt wurde}
  hoehe=600;

var schrift: integer;

begin
  schrift := pixelsperinch-96;
  if screen.width>breite then begin
    if schrift>0 then
      {fr groe Schriften}
      ScaleBy(round(((screen.width/breite)*100)-((screen.width/breite-1)*100)),100)
    else
      {fr kleine Schriften}
      ScaleBy(round(((screen.width/breite)*100)),100);
  end
  else begin
    if screen.width=breite then
      ScaleBy(round(((screen.width/breite)*100)-schrift),100)
    else begin
      if schrift>0 then
        {fr groe Schriften}
        ScaleBy(round((100-(breite/screen.width-1)*100)-(breite/screen.width-1)*100),100)
      else
        {fr kleine Schriften}
        ScaleBy(round((100-(breite/screen.width-1)*100)),100);
    end;
  end;
end;

procedure TFO_I2C.PB_CheckCableClick(Sender: TObject);
begin
IF CheckCable then
  showmessage('Cable OK')
else
  showmessage('Cable Not OK')

end;

procedure TFO_I2C.PB_ReadEEPromClick(Sender: TObject);
  var data,i,j :Integer;
      StrH1,StrH2:String;
begin
  ME_EEProm.Lines.Clear;
  iic_start(DelayTime);
  iic_w(160);
  j:=0;
  if iic_ack then
  begin
    iic_w(0);
    if iic_ack then
    Begin
      SB1.Panels[0].Text:='OK';
      iic_stop;
      iic_start(DelayTime);
      iic_w(161);
      for i:=0 to 254 do
      Begin
        J:=j+1;
        data:=iic_rack;
        Case RG_EEPromFormat.ItemIndex of
          0 : strh2:=Inttostr(data);
          1 : strh2:=IntToHexa(data,2)
        else
          strh2:=Inttostr(data); (* Default *)
        end; (* Case *)
        strh1:=Strh1 + strh2 + ' ';

        if j=8 Then
        Begin
          j:=0;
          ME_EEProm.Lines.Add(Strh1);
          strh1:='';
        end;
      end;
      data:=iic_rnak;
      ME_EEProm.Lines.Add(Inttostr(data));
      iic_stop;
    end
    else
      SB1.Panels[0].Text:='FEHLER beim Schreiben der Daten';

  end
  else
  begin
    SB1.Panels[0].Text:='FEHLER beim Schreiben der Adresse';
    iic_stop;
  end;
  //showmessage(Inttostr(data));
end;

procedure TFO_I2C.PB_WriteEEPromClick(Sender: TObject);
begin
  iic_start(DelayTime);
  iic_w(160);
  if iic_ack then
  begin
    iic_w(0);  (* Adress 0 *)
    if iic_ack then
    Begin
      SB1.Panels[0].Text:='OK';
     // iic_w(x);
     { For i:=0 to 10 do
      Begin
          iic_w(i);
          iic_w(i);
      end; {}
      iic_w(1);
      iic_w(1);
      iic_w(1);
      iic_w(1);
      iic_stop;
      iic_start(DelayTime);
      iic_w(160);
      iic_w(4);  (* Adress 0 *)
      iic_w(4);
      iic_w(4);
      iic_w(4);
      iic_stop;
    end
    else
      SB1.Panels[0].Text:='FEHLER beim Schreiben der Daten';

  end
  else
  begin
    SB1.Panels[0].Text:='FEHLER beim Schreiben der Adresse';
    iic_stop;
  end;
  //showmessage(Inttostr(data));

end;

procedure TFO_I2C.PB_SaveContentsOfEEPROMClick(Sender: TObject);
begin
  if SD1.Execute then
    ME_EEProm.Lines.SaveToFile(SD1.FileName);
end;

Procedure TFO_I2C.CalcSpecialSigns(var byte1,byte2:Integer);
  var i:integer;
begin
  byte1:=0;
  byte2:=0;
  For i:=0 to 7 do
  Begin
    if CLB_SpecialSigns1.Checked[i] then
         byte1:=byte1+zweierpotenz(i);
    if CLB_SpecialSigns2.Checked[i] then
         byte2:=byte2+zweierpotenz(i)
  end;
end;  (* CalcSpecialSigns *)

procedure TFO_I2C.PB_SendToDisplayClick(Sender: TObject);
 var err,i,adr,ih1,charval, iMaxChar,iSC1,iSC2:integer;

begin
  val('$'+CB_AdrDisplay.Text,adr,err);
  //showmessage(Inttostr(adr));
  CalcSpecialSigns(iSC1,iSC2);
  iic_start(Delaytime);
  iic_w(adr);
  if iic_ack then
  Begin
    iic_w(224);  (* Device select *)
    (* Load 11100000B, means: C=1 (another Command is following)
                              device select: A0 - A2 = 0 *)
    ih1:=192 + RG_DriveMode.ItemIndex + (4*RG_Bias.ItemIndex) +
          (8*RG_Display.ItemIndex) + (16*RG_PowerMode.ItemIndex);

    if iic_ack then
    (* Load 11001000B, means: C=1
                              LP=1 (Chip on)
                              E=0 (Display enable)
                              B=0 (1/3 bias)
                              M=00 (mux 1:4, 4 BP) *)
       iic_w(ih1);
    ih1:=240 + (RG_BlinkMode.ItemIndex * 4) + RG_DisplayBlink.ItemIndex;
    //showmessage(inttostr(ih1));
    if iic_ack then
      iic_w(ih1);
       (* 011100xx  = 112
                    11110xxx =240 + xx *)
    if iic_ack then
      iic_w(0);  (* set display to initial position *)
    if iic_ack then
    begin
      if length(ED_LCD_Displaytext.Text)<=MaxDisplayChar then
        iMaxChar:=length(ED_LCD_Displaytext.Text)
      else
        iMaxChar:=MaxDisplayChar;  (* Displaytext too long then cut (send only
                                      "MaxDisplayChar" Bytes to display) *)
      if iMaxChar<MaxDisplayChar then
        (* write empty digit *)
        for i:=iMaxChar+1 to MaxDisplayChar do
        begin
          iic_w(0);
          iic_w(0);
        end;
      For i:=1 to iMaxChar do
      Begin
        (* Offset is 32 (20h): first char is "space" (ASCIICode 32) *)
        charval:=ord(ED_LCD_Displaytext.Text[i])-32;
        //showmessage(inttostr(charval));
        iic_w(DisplayChar[charval].part1); (* Byte 1 *)
        iic_w(DisplayChar[charval].part2); (* Byte 2 *)
      end;
      iic_w(iSC1);   (* special signs of the display *)
      iic_w(iSC2);
    end;
  end  (* Ack after sending adress *)
  else
    Showmessage('No Ack after sending adress!');
  iic_stop;
end;
{iic_w(0); (* 1 *)
    iic_w(9);
    iic_w(153); (* 2 *)
    iic_w(153);
    iic_w(57);   (* 3 *)
    iic_w(156);
    iic_w(57);   (* 4 *)
    iic_w(156);
    iic_w(57);   (* 5 *)
    iic_w(156);
    iic_w(57);   (* 6 *)
    iic_w(156);
    iic_w(57);   (* 7 *)
    iic_w(156);
    iic_w(0);   (* 8 *)
    iic_w(9);


    }
procedure TFO_I2C.CLB_SpecialSigns1Click(Sender: TObject);
  Var iSS1,iSS2:Integer;
begin
  CalcSpecialSigns(iSS1,iSS2);
  LB_Specialsignval1.Caption:=Inttostr(iSS1);
  LB_Specialsignval2.Caption:=Inttostr(iSS2);
end;

procedure TFO_I2C.Anschalten1Click(Sender: TObject);
begin
  TS_Display.Enabled:=true
end;

procedure TFO_I2C.PB_ReadTempClick(Sender: TObject);
 var err,adr,lsb,msb:integer;
     ActTemp:Real;
(*  P0-P1: Register Select:
    P1 P0 Register
    0 0 Temperature (Read only) (Power-up default)
    0 1 Configuration (Read/Write)
    1 0 THYST (Read/Write)
    1 1 TOS (Read/Write)
*)

begin
  val('$'+CB_AdrTemp.Text,adr,err);
  iic_start(Delaytime);
  iic_w(adr);  (* Adressbyte *)
  iic_w(0);    (* Pointerbyte: 0 means read actual temperature *)
  iic_start(Delaytime);
  iic_w(adr+1);  (* repeat start *)
  msb:=iic_rack();
  lsb:=iic_rnak();
  iic_stop;
  ActTemp:=CalcTemperature(msb,lsb);
  DA_Temp.Value:=FloatToStr(ActTemp) + 'C';
end;

function TFO_I2C.CalcLM75ConfigByte:Integer;
  var i,tmpByte:integer;
Begin
  tmpByte:=0;
  for i:=0 to 2 do
    if CLB_TempConfig.Checked[i] then
         tmpByte:=tmpByte+zweierpotenz(i);
  i:=3;
  if (RG_TempFaultQueue.ItemIndex = 1) or
     (RG_TempFaultQueue.ItemIndex = 3) then
     tmpByte:=tmpByte+zweierpotenz(i);
  i:=i+1;
  if (RG_TempFaultQueue.ItemIndex) >=2 then
     tmpByte:=tmpByte+zweierpotenz(i);
  CalcLM75ConfigByte:=tmpByte
end;

procedure TFO_I2C.PB_WriteTempConfigClick(Sender: TObject);
  var err,adr,dat:integer;

(*  P0-P1: Register Select:
    P1 P0 Register
    0 0 Temperature (Read only) (Power-up default)
    0 1 Configuration (Read/Write)
    1 0 THYST (Read/Write)
    1 1 TOS (Read/Write)
*)

begin
  dat:=CalcLM75ConfigByte;
  LB_Configbyte.Caption:=IntToStr(dat) + ' / ' +
                          IntToBinary(dat,8);
  val('$'+CB_AdrTemp.Text,adr,err);
  iic_start(Delaytime);
  iic_w(adr);  (* Adressbyte *)
  iic_w(1);    (* Pointerbyte: 1 means Configuration Byte *)
  iic_w(dat);  (* write calculated data *)
  iic_stop;
end;

procedure TFO_I2C.PB_ReadAlarmTempClick(Sender: TObject);
  var err,adr,lsb,msb:integer;
     ActTemp:Real;
(*  P0-P1: Register Select:
    P1 P0 Register
    0 0 Temperature (Read only) (Power-up default)
    0 1 Configuration (Read/Write)
    1 0 THYST (Read/Write)
    1 1 TOS (Read/Write)
*)

begin
  val('$'+CB_AdrTemp.Text,adr,err);
  iic_start(Delaytime);
  iic_w(adr);  (* Adressbyte *)
  iic_w(3);    (* Pointerbyte: 3 means read TOS *)
  iic_start(Delaytime);
  iic_w(adr+1);  (* repeat start *)
  msb:=iic_rack();
  lsb:=iic_rnak();
  iic_stop;
  ActTemp:=CalcTemperature(msb,lsb);
  ED_AlarmTemp.Text:=FloatToStr(ActTemp);
end;

procedure TFO_I2C.PB_ReadTHystClick(Sender: TObject);
  var err,adr,lsb,msb:integer;
     ActTemp:Real;
(*  P0-P1: Register Select:
    P1 P0 Register
    0 0 Temperature (Read only) (Power-up default)
    0 1 Configuration (Read/Write)
    1 0 THYST (Read/Write)
    1 1 TOS (Read/Write)
*)

begin
  val('$'+CB_AdrTemp.Text,adr,err);
  iic_start(Delaytime);
  iic_w(adr);  (* Adressbyte *)
  iic_w(2);    (* Pointerbyte: 2 means read   var err,adr,lsb,msb:integer;  *)
  iic_start(Delaytime);
  iic_w(adr+1);  (* repeat start *)
  msb:=iic_rack();
  lsb:=iic_rnak();
  iic_stop;
  ActTemp:=CalcTemperature(msb,lsb);
  ED_ClearTemp.Text:=FloatToStr(ActTemp);
end;

Procedure TFO_I2C.SetDesiredTemp (TempType:Integer; ValueToWrite:Integer);
(* TempType is either 2 or 3 to select, which temperture is to write  *)
  var err,adr:integer;
Begin
  if (ValueToWrite<-55) or (ValueToWrite>125) then
  Begin
    ValueToWrite := 80;          (* check the limits of the value *)

  end;
  if ValueToWrite<0 then (* handle negative values *)
    ValueToWrite:=Abs(ValueToWrite)+128;
  val('$'+CB_AdrTemp.Text,adr,err);
  iic_start(Delaytime);
  iic_w(adr);  (* Adressbyte *)
  iic_w(TempType);
  iic_w(ValueToWrite);
  iic_w(0);     (* lsb will not be used at the moment *)
  iic_stop;
  if TempType=3 then
    LB_AlarmMSB.Caption:=IntToStr(ValueToWrite) + ' / ' +
                  IntToBinary(ValueToWrite,8);
  if TempType=2 then
    LB_THYSTMSB.Caption:=IntToStr(ValueToWrite) + ' / ' +
                  IntToBinary(ValueToWrite,8);
  LB_AlarmLSB.Caption:=IntToStr(0) + ' / ' +
                          IntToBinary(0,8);
  LB_THYSTLSB.Caption:=IntToStr(0) + ' / ' +
                          IntToBinary(0,8);
end;

procedure TFO_I2C.PB_AlarmTempWriteClick(Sender: TObject);
  var msb:integer;
begin
  msb:=StrToIntDef(ED_AlarmTemp.Text,80);
  SetDesiredTemp(3,msb);  (* Pointerbyte: 3 means adressing Alarm Temperature *)
end;

procedure TFO_I2C.PB_WriteTHystClick(Sender: TObject);
  var msb:integer;
begin
  msb:=StrToIntDef(ED_ClearTemp.Text,80);
  SetDesiredTemp(2,msb);    (* Pointerbyte: 2 means adressing THyst *)
end;

procedure TFO_I2C.CB_ContTempReadClick(Sender: TObject);
begin
  if CB_ContTempRead.Checked then
  Begin
     TI_TemperatureRead.Enabled:=true;
     PB_ReadTemp.Enabled:=False;
  end
  else
  Begin
    TI_TemperatureRead.Enabled:=false;
    PB_ReadTemp.Enabled:=True
  end
end;

procedure TFO_I2C.TI_TemperatureReadTimer(Sender: TObject);
begin
  PB_ReadTemp.Click;
end;

procedure TFO_I2C.CB_RunWithoutCheckClick(Sender: TObject);
begin
  if CB_RunWithoutCheck.Checked then
    Tabsheet_Handling(True) (* Enable Tabsheets *)
  else
    PB_LookfordevicesClick(Sender);
end;

end.





