(*****************************************************************************
*** Name of the Unit:      I2Csio
*** Function:              I2C-Interface at the serial port
*** Author:                Andreas Hesse and others (see V1.0.0.0 in Version
***                        Version Management)
*** Date of construction:  2002-01-24
*** Date of last revision: 2002-02-21
*** Version:               1.0.1.0
*** Comment:
***
***
***
******************************************************************************
***
*** + = Adding new feature
*** - = Bugfix
*** * = Changed
*** ? = detected problem
***
******************************************************************************
***
*** Version management:
***
*** V 1.0.1.1   + CheckCable function to verify lines
*** V 1.0.1.0   * Code Review (Cosmetic changes)
*** V 1.0.0.1   Changes: Replace Assembler-Routines with API-Functions
***             	Program should run then under WINNT/W2K
***             	Constants: MS_CTS_ON, MS_DSR_ON, MS_RING_ON,  MS_RLSD_ON
*** V 1.0.0.0   First construction based on several sources coming from the
***             	internet
***             	Thanks to:
***             	Robert-Bosch-Schule Ulm
***             	Abt. Elektrotechnik, Datentechnik, Programmiersprachen
*** *)


unit i2csiou;

interface

uses windows;

const maxcom=2;

var
  iic_ack: boolean;
  (* baseadr: smallint=$3F8;      3F8=COM1 2F8=COM2
    will be needed when using the assembler routines *)
  delaytime:Longint;
  FhComm:THandle;

procedure iic_sda_0;                   	(*  SDA auf LOW  setzen *)
procedure iic_sda_1;                   	(*  SDA auf HIHG setzen *)
procedure iic_scl_0;                    (*  SCL auf LOW  setzen *)
procedure iic_scl_1;                    (*  SCK auf HIGH setzen *)

function iic_sda:boolean;               (*  Zustand von SDA abfragen *)
function iic_scl:boolean;               (*  Zustand von SCL abfragen *)

procedure iic_start(dt:longint);        (*  I2C-Startbedingung *)
procedure iic_stop;                     (*  I2C-Stopbedingung *)

procedure iic_w(b:byte);                (*  ein Byte auf Bus ausgeben *)

function iic_rack:byte;                 (*  ein Byte lesen mit  ACKNOWLEDGE *)
function iic_rnak:byte;                 (*  ein Byte lesen ohne ACKNOWLEDGE *)

function CheckCable:Boolean;            (* Check, if a cable is connected *)

procedure OpenCom(COMPort:Integer);
																	      (* open a COM under Windows *)
procedure CloseCom;                    	(* close a COM under Windows *)


implementation


{procedure Out32(PortAddress:smallint;Value:smallint);
  var ByteValue:Byte;
begin
  ByteValue:=Byte(Value);
  asm
    push dx
    mov dx,PortAddress
    mov al, ByteValue
    out dx,al
    pop dx
  end;
end;

function Inp32(PortAddress:smallint):smallint;
  var ByteValue:byte;
begin
  asm
    push dx
    mov dx, PortAddress
    in al,dx
    mov ByteValue,al
    pop dx
  end;
  Inp32:=smallint(ByteValue) and $00FF;
end;        }

(* Wait: I2C-Timing *)

procedure wait(w:longint);
	var i,j:longint;
begin
	j:=0;
	for i:=0 to 100*w do
		inc(j);
end;

procedure iic_sda_0;
begin
	(* force DTR Line to 0 *)
	(* out32(baseadr+4,(inp32(baseadr+4)and  $FE));
	  old assembler routines *)
	EscapeCommFunction(FhComm,CLRDTR);
	wait(delaytime);
end;

procedure iic_sda_1;
begin
	(* force DTR Line to 1 *)
	(* out32(baseadr+4,(inp32(baseadr+4)or $01));
	  old assembler routines *)
	EscapeCommFunction(FhComm,SETDTR);
	wait(delaytime);
end;

procedure iic_scl_0;
begin
	(* force RTS Line to 0 *)
	(* out32(baseadr+4,(inp32(baseadr+4)and $FD));
	  old assembler routines *)
	EscapeCommFunction(FhComm,CLRRTS);
	wait(delaytime);
end;

procedure iic_scl_1;
begin
	(* force RTS Line to 1 *)
	(* out32(baseadr+4,(inp32(baseadr+4) or $02));
	  old assembler routines *)
	EscapeCommFunction(FhComm,SETRTS);
	wait(delaytime);
end;

function iic_sda:boolean;
  var Status:Cardinal;
begin
	(* get Status of CTS Line *)
	(* iic_sda:=(inp32(baseadr+6)and $10)=$10;
	  old assembler routines *)
	GetCommModemStatus(FhComm,Status);
	iic_sda:=(Status and MS_CTS_ON)=MS_CTS_ON;
	wait(delaytime);
end;

function iic_scl:boolean;
  var Status:Cardinal;
begin
	(* get Status of DCD Line *)
	(* iic_scl:=(inp32(baseadr+6)and $80)=$80;
	  old assembler routines *)
	GetCommModemStatus(FhComm,Status);
	iic_scl:=(Status and MS_RLSD_ON)=MS_RLSD_ON;
	wait(delaytime);
end;

procedure iic_start(dt:longint);
begin
	Delaytime:=dt;
	iic_scl_0;
	iic_sda_1;
	iic_scl_1;
	iic_sda_0;
	iic_scl_0;
end;

procedure iic_stop;
begin
	iic_scl_0;
	iic_sda_0;
	iic_scl_1;
	iic_sda_1;
end;

procedure iic_w1;
begin
  iic_scl_0;
  iic_sda_1;
  iic_scl_1;
  iic_scl_0;
end;

procedure iic_w0;
begin
	iic_scl_0;
  iic_sda_0;
  iic_scl_1;
  iic_scl_0;
end;

procedure iic_w(b:byte);
  var n,mask:byte;
begin
  mask:=128;
  for n:=0 to 7 do
  begin
	  if (mask and b) = mask then
      iic_w1
    else
      iic_w0;
    mask:=mask div 2;
	end;
	iic_sda_1;
  iic_scl_1;
  iic_ack:= not iic_sda;
  iic_scl_0;
  iic_sda_0;
end;

function iic_rbit:boolean; (* ein BIT vom IIC-BUS lesen *)
begin
	iic_scl_0;
	iic_sda_1;
	iic_scl_1;
	iic_rbit:=iic_sda;
	iic_scl_0;
	iic_sda_0;
end;

function iic_rack:byte;
	var bit,da,ma:byte;
begin
	ma:=128;
  da:=0;
	for bit:=7 downto 0 do
	begin
		if iic_rbit then
      da:=da+ma;
    ma:=ma div 2;
	end;
	iic_rack:=da;
  iic_w0; (* ACK an Sender zurckschicken *)
end;

function iic_rnak:byte;
  var bit,da,ma:byte ;
begin
	ma:=128;
  da:=0;
	for bit:=7 downto 0 do
	begin
		if iic_rbit then
      da:=da+ma;
    ma:=ma div 2;
	end;
	iic_rnak:=da;
  iic_w1; (* Kein ACK an Sender zurckschicken *)
end;

(* Check function of SCL and SDA
    Return: False, if one line (SDA or SCL) is not connected
            or no power at the circuits
            or no circuits available *)

function CheckCable:Boolean;
  var sclOK,sdaOK:Boolean;
Begin
  sclOK:=False;
  sdaOK:=False;
  iic_sda_0;
  if not iic_sda then
  Begin
    iic_sda_1;
    if iic_sda then
      sdaOK:=True;
  end;
  iic_scl_0;
  if not iic_scl then
  Begin
    iic_scl_1;
    if iic_scl then
      sclOK:=True;
  end;
  iic_sda_0;
  iic_scl_0;
  CheckCable:=sclOK and sdaOK
end;

procedure OpenCom(COMPort:Integer);
  var sh1,SComString:string;
Begin
	IF (COMPort>0) and (COMPort<=MaxCom) then
	(* This decision is not really necessary, when filtering COMs with the COMCB-box
	But for safety reason it is OK *)
	begin
		STR(COMPort,SH1);
		SCOMString:='COM'+SH1;
		FhComm:=CreateFile(Pchar(SCOMString),Generic_READ or Generic_WRITE,
							  0,nil,OPEN_EXISTING,FILE_FLAG_OVERLAPPED,0)
	end;
end;

procedure CloseCom;
Begin
	CloseHandle(FhComm);
end;

end.
